"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class Flex {
    /**
     * Handles:
     * - synchronous values and callbacks;
     * - asynchronous values and callbacks.
     */
    static get(value, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const onError = options && typeof options.onError === 'function' ? options.onError : null;
            const name = options && typeof options.name === 'string' ? options.name : undefined;
            const cc = options && options.cc;
            const p = value;
            if (p && typeof p.catch === 'function') {
                if (onError) {
                    return p.catch(err => onError(err, name));
                }
                return p;
            }
            if (typeof value !== 'function') {
                return value;
            }
            if (!onError) {
                return value.call(cc);
            }
            try {
                const v = value.call(cc);
                if (v && typeof v.catch === 'function') {
                    // tslint is buggy here: https://github.com/palantir/tslint/issues/4824
                    return v.catch(err => onError(err, name));
                }
                return v;
            }
            catch (e) {
                return onError(e, name);
            }
        });
    }
    /**
     * Handles synchronous values and callbacks.
     * Reports error on everything asynchronous.
     */
    static getSync(value, options) {
        const onError = options && typeof options.onError === 'function' ? options.onError : null;
        const name = options && typeof options.name === 'string' ? options.name : undefined;
        const cc = options && options.cc;
        const checkAsync = (v) => {
            if (v && typeof v.then === 'function') {
                const err = new Error(`Value ${name ? '"' + name + '" ' : ''}cannot be asynchronous.`);
                if (onError) {
                    return onError(err, name);
                }
                throw err;
            }
            return v;
        };
        if (typeof value !== 'function') {
            return checkAsync(value);
        }
        const getValue = () => value.call(cc);
        if (!onError) {
            return checkAsync(getValue());
        }
        let v;
        try {
            v = getValue();
        }
        catch (e) {
            return onError(e, name);
        }
        return checkAsync(v);
    }
}
exports.Flex = Flex;
